from pathlib import Path

import pytest

from quom.__main__ import main

FILE_MAIN_HPP = """
int foo(); // qθομ"""


def test_file_encoding_default_encoding(fs):
    with open('main.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_MAIN_HPP)

    main(['main.hpp', 'result.hpp'])
    assert Path('result.hpp').read_text('utf-8') == FILE_MAIN_HPP

    with pytest.raises(UnicodeDecodeError):
        Path('result.hpp').read_text('ascii')

    with pytest.raises(UnicodeDecodeError):
        Path('result.hpp').read_text('utf-32')


def test_file_encoding_custom_encoding(fs):
    with open('main.hpp', 'w+', encoding='utf-32') as file:
        file.write(FILE_MAIN_HPP)

    main(['main.hpp', 'result.hpp', '--encoding=utf-32'])

    assert Path('result.hpp').read_text('utf-32') == FILE_MAIN_HPP

    with pytest.raises(UnicodeDecodeError):
        Path('result.hpp').read_text('utf-8')

    with pytest.raises(UnicodeDecodeError):
        main(['main.hpp', 'result.hpp'])

    with pytest.raises(UnicodeDecodeError):
        main(['main.hpp', 'result.hpp', '--encoding=utf-8'])



import os
from io import StringIO

from quom import Quom

FILE_FOO_HPP = """
#include "../b/bar.hpp"
"""

FILE_BAR_HPP = """
int foo();
"""

FILE_MAIN_CPP = """
#include "a/foo.hpp"
#include "b/bar.hpp"
"""

RESULT = """

int foo();
"""


def test_same_file_different_include(fs):
    os.makedirs('a')
    os.makedirs('b')

    with open('main.cpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_MAIN_CPP)

    with open('a/foo.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_FOO_HPP)

    with open('b/bar.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_BAR_HPP)

    dst = StringIO()
    Quom('main.cpp', dst)

    assert dst.getvalue() == RESULT



from io import StringIO
from pathlib import Path

from quom import Quom

FILE_MAIN_CPP = """\
#include "a.hpp"
int main() {
    return 0;
}
// Stitch Begin
// End
"""

FILE_A_HPP = 'int a;'

FILE_A_CPP = """\
#include "b.hpp"
#include "c.hpp"
void mid() {}"""

FILE_B_HPP = 'int b;'
FILE_C_HPP = 'int c;'

FILE_B_CPP = """\
#include <b>"""

FILE_C_CPP = """\
#include <c>"""

RESULT = """\
int a;
int main() {
    return 0;
}
// Stitch Begin
// End
int b;
int c;
void mid() {}
#include <b>
#include <c>
"""

RESULT_STITCH = """\
int a;
int main() {
    return 0;
}
int b;
int c;
void mid() {}
#include <b>
#include <c>

// End
"""


def init():
    with open('main.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_MAIN_CPP)

    with open('a.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_A_HPP)

    with open('a.cpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_A_CPP)

    with open('b.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_B_HPP)

    with open('b.cpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_B_CPP)

    with open('c.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_C_HPP)

    with open('c.cpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_C_CPP)


def test_add_line_break_in_stitched_files_if_missing(fs):
    init()

    dst = StringIO()
    Quom(Path('main.hpp'), dst)

    assert dst.getvalue() == RESULT


def test_add_line_break_in_stitched_files_if_missing_at_stitch_location(fs):
    init()

    dst = StringIO()
    Quom(Path('main.hpp'), dst, stitch_format='Stitch Begin')

    assert dst.getvalue() == RESULT_STITCH



from pathlib import Path

from quom.__main__ import main

FILE_FOO_HPP = """
int VAR_NAME;
#undef VAR_NAME
"""

FILE_MAIN_CPP = """
#define VAR_NAME a
#include "foo.hpp"
#define VAR_NAME b
#include "foo.hpp"
"""

RESULT = """
#define VAR_NAME a

int VAR_NAME;
#undef VAR_NAME

#define VAR_NAME b

int VAR_NAME;
#undef VAR_NAME

"""


def test_file_without_include_guard(fs):
    with open('main.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_MAIN_CPP)

    with open('foo.hpp', 'w+', encoding='utf-8') as file:
        file.write(FILE_FOO_HPP)

    main(['main.hpp', 'result.hpp'])
    assert Path('result.hpp').read_text() == RESULT
